table_device_view.cuh
Go to the documentation of this file.
1 /*
2  * SPDX-FileCopyrightText: Copyright (c) 2019-2026, NVIDIA CORPORATION & AFFILIATES. All rights reserved.
3  * SPDX-License-Identifier: Apache-2.0
4  */
5 #pragma once
6 
9 #include <cudf/types.hpp>
12 
13 #include <rmm/cuda_stream_view.hpp>
14 #include <rmm/device_buffer.hpp>
15 #include <rmm/resource_ref.hpp>
16 
17 #include <cassert>
18 #include <memory>
19 
25 namespace CUDF_EXPORT cudf {
26 namespace detail {
27 
44 template <typename ColumnDeviceView, typename HostTableView>
46  public:
47  table_device_view_base() = delete;
48  ~table_device_view_base() = default;
63 
69  __device__ ColumnDeviceView* begin() const noexcept { return _columns; }
70 
79  __device__ ColumnDeviceView* end() const noexcept { return _columns + _num_columns; }
80 
87  __device__ ColumnDeviceView const& column(size_type column_index) const noexcept
88  {
89  assert(column_index >= 0);
90  assert(column_index < _num_columns);
91  return _columns[column_index];
92  }
93 
100  __device__ ColumnDeviceView& column(size_type column_index) noexcept
101  {
102  assert(column_index >= 0);
103  assert(column_index < _num_columns);
104  return _columns[column_index];
105  }
106 
112  [[nodiscard]] __host__ __device__ size_type num_columns() const noexcept { return _num_columns; }
113 
119  [[nodiscard]] __host__ __device__ size_type num_rows() const noexcept { return _num_rows; }
120 
127  void destroy();
128 
129  private:
130  ColumnDeviceView* _columns{};
131  size_type _num_rows{};
132  size_type _num_columns{};
133 
134  protected:
142  table_device_view_base(HostTableView source_view, ColumnDeviceView* columns);
143 };
144 } // namespace detail
145 
149 class table_device_view : public detail::table_device_view_base<column_device_view, table_view> {
150  public:
166  static std::unique_ptr<table_device_view, std::function<void(table_device_view*)>> create(
167  table_view source_view,
170 
171  private:
172  table_device_view(table_view source_view, column_device_view* columns);
173 };
174 
181  : public detail::table_device_view_base<mutable_column_device_view, mutable_table_view> {
182  public:
198  static std::unique_ptr<mutable_table_device_view, std::function<void(mutable_table_device_view*)>>
202 
203  private:
205 };
206 
217 template <typename ColumnDeviceView, typename HostTableView>
218 std::pair<std::unique_ptr<rmm::device_buffer>, ColumnDeviceView*> create_column_device_views(
219  HostTableView source_view,
220  rmm::cuda_stream_view stream,
222 
223 } // namespace CUDF_EXPORT cudf
An immutable, non-owning view of device data as a column of elements that is trivially copyable and u...
Base class for a device table of ColumnDeviceViews.
size_type num_rows() const noexcept
Returns the number of rows.
ColumnDeviceView const & column(size_type column_index) const noexcept
Returns a reference to the view of the specified column.
table_device_view_base(table_device_view_base &&)=default
Move constructor.
ColumnDeviceView * begin() const noexcept
Returns an iterator to the first view in the table.
void destroy()
Destroy the table_device_view object.
table_device_view_base(table_device_view_base const &)=default
Copy constructor.
table_device_view_base(HostTableView source_view, ColumnDeviceView *columns)
Constructor for subclasses to create a table device view from a host table view.
size_type num_columns() const noexcept
Returns the number of columns.
table_device_view_base & operator=(table_device_view_base const &)=default
Copy assignment operator.
ColumnDeviceView * end() const noexcept
Returns an iterator one past the last column view in the table.
table_device_view_base & operator=(table_device_view_base &&)=default
Move assignment operator.
ColumnDeviceView & column(size_type column_index) noexcept
Returns a reference to the view of the specified column.
A mutable, non-owning view of device data as a column of elements that is trivially copyable and usab...
Mutable table device view that is usable in device memory.
static std::unique_ptr< mutable_table_device_view, std::function< void(mutable_table_device_view *)> > create(mutable_table_view source_view, rmm::cuda_stream_view stream=cudf::get_default_stream(), rmm::device_async_resource_ref mr=cudf::get_current_device_resource_ref())
Factory to construct a mutable table device view that is usable in device memory.
A set of mutable_column_views of the same size.
Definition: table_view.hpp:274
Table device view that is usable in device memory.
static std::unique_ptr< table_device_view, std::function< void(table_device_view *)> > create(table_view source_view, rmm::cuda_stream_view stream=cudf::get_default_stream(), rmm::device_async_resource_ref mr=cudf::get_current_device_resource_ref())
Factory to construct a table device view that is usable in device memory.
A set of cudf::column_view's of the same size.
Definition: table_view.hpp:206
Column device view class definitions.
APIs for querying the default CUDA stream and per-thread default stream status.
rmm::cuda_stream_view const get_default_stream()
Get the current default stream.
rmm::device_async_resource_ref get_current_device_resource_ref()
Get the current device memory resource reference.
cuda::mr::resource_ref< cuda::mr::device_accessible > device_async_resource_ref
int32_t size_type
Row index type for columns and tables.
Definition: types.hpp:76
APIs for getting and setting the current device memory resource.
cuDF interfaces
Definition: host_udf.hpp:27
std::pair< std::unique_ptr< rmm::device_buffer >, ColumnDeviceView * > create_column_device_views(HostTableView source_view, rmm::cuda_stream_view stream, rmm::device_async_resource_ref mr=cudf::get_current_device_resource_ref())
Copies the contents of a table_view to a column device view in contiguous device memory.
Class definitions for (mutable)_table_view
Type declarations for libcudf.